/* Copyright (C) 1994 Queen Mary and Westfield College, University of London

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   A copy of the GNU General Public License may be found in
   doc/COPYING in the source tree of this programm.

   This file was written by: Franz Gerschwiler.(goblin@dcs.qmw.ac.uk)

 ****

   modified by: p.hlstede	(peter@calvados.chemie.uni-bremen.de)

   - the code was reorganized and ansified

							06.09.1997
 */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <errno.h>

#include <Xm/Xm.h>
#include <Xm/Text.h>
#include <Xm/List.h>

#include "sys_consts.h"
#include "globals.h"
#include "utils.h"
#include "callbacks.h"
#include "popUps.h"
#include "helpEvents.h"
#include "dstring.h"


/***  PROCEDURE DECLARATIONS  ***/
static Boolean loadHelpFile(char *filename);


/* Main help facility */
void  gui_helpCB(Widget w, XtPointer client_data, XtPointer call_data)
{
   XmString item;

   set_status_message("Ok, running Help");

   /* Decide what called help */
   switch ((HelpReason)client_data)
   {
      case HELPCALL:
	 if (helpPopUp == (Widget)0)
	    gui_createHelpPopUp(toplevel);
	 XtManageChild(helpPopUp);
	 break;

      case HELPCANCEL:
	 XtUnmanageChild(helpPopUp);
	 set_status_message("");
	 break;

      case HELPHELP:		/* Help called from help popUp */
	 item = XmStringCreateLtoR("helpOnHelp", CHARSET);
	 XmListSelectItem(helpLI, item, True);
	 break;

      case HELPSETUP:		/* Help called from setup popUp */
	 if (helpPopUp == (Widget) 0)
	    gui_createHelpPopUp(toplevel);
	 XtManageChild(helpPopUp);
	 item = XmStringCreateLtoR("SetUp", CHARSET);
	 XmListSelectItem(helpLI, item, True);
	 break;

      case HELPABOUT:
	 gui_about();
	 break;

      default:
	 break;
   }
}

/* Deals with user selecting a help topic */
void  gui_helpSelectCB(Widget w, XtPointer client_data, XtPointer call_data)
{
   XmListCallbackStruct *cbd = (XmListCallbackStruct *)call_data;
   char    *curr_item;

   XmStringGetLtoR(cbd->item, CHARSET, &curr_item);
   loadHelpFile(curr_item);
   XtFree(curr_item);
}


/* Read in help index to create list of topics */
void  gui_createHelpList(void)
{
   XmString item;
   struct stat stat_val;
   char    *filecontents;
   FILE    *fpointer;
   int      f_length;
   char    *idstring;
   DString  filename;
   int      count;

   filecontents = NULL;
   DStringCompose(&filename, HELPPATH, "/gloeHelpIndex", NULL);
   if (stat(filename.str, &stat_val) == 0)
   {
      f_length = stat_val.st_size;
      fpointer = fopen(filename.str, "r");
      if (fpointer != NULL)
      {
	 filecontents = XtMalloc((unsigned)(f_length + 1));
	 *filecontents = '\0';
	 fread(filecontents, sizeof(char), f_length, fpointer);
	 filecontents[f_length] = '\0';
	 fclose(fpointer);
      }
      if (filecontents != NULL)
      {
	 count = 1;
	 XmListDeselectAllItems(helpLI);
	 idstring = strtok(filecontents, SEPCHARS);
	 while (idstring != NULL)
	 {
	    item = XmStringCreateLtoR(idstring, CHARSET);
	    XmListAddItemUnselected(helpLI, item, count);
	    idstring = strtok(NULL, SEPCHARS);
	    count++;
	 };
	 XmListSelectPos(helpLI, 1, True);
	 XtFree(filecontents);
      }
   }
   else
      gui_error("Sorry unable to load help data. Please contact sys admin.");
   DStringFree(&filename);
}


/* Read in a help file */
static Boolean loadHelpFile(char *filename)
{
   struct stat stat_val;
   char    *filecontents;
   FILE    *fpointer;
   int      f_length;
   DString  file;
   Boolean  result = False;

   DStringCompose(&file, HELPPATH, "/", filename, NULL);
   if (stat(file.str, &stat_val) == 0)
   {
      f_length = stat_val.st_size;
      fpointer = fopen(file.str, "r");
      if (fpointer != NULL)
      {
	 filecontents = XtMalloc((unsigned)(f_length + 1));
	 *filecontents = '\0';
	 fread(filecontents, sizeof(char), f_length, fpointer);
	 filecontents[f_length] = '\0';
	 fclose(fpointer);
	 XmTextSetString(helpText, filecontents);
	 result = True;
      }
   }
   DStringFree(&file);
   return(result);
}

void  gui_contextSensitiveHelpCB(Widget w, XtPointer client_data, XtPointer call_data)
{
   char  *p;

   /* Decide what called help */
   switch ((HelpItem)client_data)
   {
      case SAVE:
	 p = "Save the active file under current name";
	 break;

      case SAVEAS:
	 p = "Save the active file under a new name";
	 break;

      case SAVESELECTION:
	 p = "Save selected text";
	 break;

      case OPEN:
	 p = "Open a file";
	 break;

      case INSERT:
	 p = "Insert a text file in the current text";
	 break;

      case NEW:
	 p = "Obtain a blank document";
	 break;

      case PRINT:
	 p = "Print the active document";
	 break;

      case PRINTSELECTION:
	 p = "Print selected text";
	 break;

      case QUIT:
	 p = "Exit gloe";
	 break;

      case COPY:
	 p = "Copy selected text to clipboard";
	 break;

      case CUT:
	 p = "Remove selected text and copy to clipboard";
	 break;

      case PASTE:
	 p = "Insert text from clipboard";
	 break;

      case UNDO:
	 p = "Undo last edit";
	 break;

      case REDO:
	 p = "Undo last undo action";
	 break;

      case SEARCH:
	 p = "Search for text string in document";
	 break;

      case REPLACE:
	 p = "Replace text string found by search";
	 break;

      case REPLACEALL:
	 p = "Replace all instances of search string with replace string";
	 break;

      case SETUP:
	 p = "Invoke set up dialog";
	 break;

      case HELPB:
	 p = "Invoke help dialog";
	 break;

      case JUMP:
	 p = "Jump to line number";
	 break;

      case JUMPTEXT:
	 p = "Enter line to jump to here";
	 break;

      case SEARCHTEXT:
	 p = "Enter search string here";
	 break;

      case REPLACETEXT:
	 p = "Enter replace string here";
	 break;

      case MATCH:
	 p = "Match parentheses characters";
	 break;

      case SELECTIONONLY:
	 p = "Set search and replace with selection only";
	 break;

      default:
         p = "";
	 break;
   }
   set_status_message(p);
}

void  gui_undoContextSensitiveHelpCB(Widget w, XtPointer client_data, XtPointer call_data)
{
   set_status_message("");
}
